DOC
(*
comp.lang.forth #10157
From: Tom Zimmer 
Subject: Re: 32-bit Forth for Win 95?
Date: Thu Mar 14 23:06:30 MET 1996
Organization: Netcom
Lines: 284
Mime-Version: 1.0
Content-Type: text/plain
Content-Transfer-Encoding: 7bit
X-NETCOM-Date: Thu Mar 14  7:59:07 PM PST 1996
X-Mailer: Mozilla 2.0 (Macintosh; I; PPC)

Here are the results of the FORTH, Inc. benchmarks as modified for Win32Forth

Testing the Win32Forth system                   WinNT
                                                P5-66
Testing DO LOOP =                              0.911 us
Testing * =                                    1.963 us
Testing / =                                    3.826 us
Testing + =                                    1.452 us
Testing M* =                                   2.003 us
Testing M/ =                                   3.034 us
Testing M+ =                                   2.844 us
Testing /MOD =                                 2.324 us
Testing */ =                                   4.045 us
Testing Eratosthenes sieve 1899 Primes =      14.770 us
Testing Hoare's quick sort (reverse order) = 170.000 us

To run all tests = 22.77 sec ok

Tom Zimmer

And here is the source for the benchmarks as modified for Win32Forth, including
the header from Elizabeth Rather;

------------------------------------------------------------------------------

Dennis Ruffer and the folks at FORTH, Inc. have a standard set of benchmarks
that we use to compare Forths.  The results of the benchmarks on ProForth for
Windows are as follows:
                                                  Win95   Win95   WinNT
                                                 486-80  486-66  486-66
Testing DO LOOP                                   0.756   0.865   0.952 us
Testing *                                         1.539   1.483   1.822 us
Testing /                                         1.895   2.184   2.233 us
Testing +                                         1.250   1.387   1.482 us
Testing M*                                        1.594   1.538   1.873 us
Testing M/                                        3.227   3.859   3.815 us
Testing M+                                        3.063   3.447   3.495 us
Testing /MOD                                      2.060   2.513   2.484 us
Testing */                                        4.587   5.411   5.498 us
Testing Erathosthenes sieve 1899 Primes          15.140  16.727  18.315 us
Testing Hoare's quick sort (reverse order)      164.000 205.000 210.000 us

To run all tests                                 20.806  23.759  24.666 sec

Source for the tests follow.

*)
ENDDOC


\ FORTH, Inc.  32 Bit Benchmark Source

4 set-precision

1 CELLS CONSTANT CELL

CELL NEGATE CONSTANT -CELL

\ : ALIGN   CELL  HERE  OVER 1- AND  - ALLOT ;

\ March 16th, 1996 - 11:15 mhx changed to use Milli-second fetch 
: COUNTER ( - n)   ?ms ;

\ March 16th, 1996 - 11:16 mhx don't have M/MOD, switched to UM/MOD
: M/ ( d n - n)   S" um/mod NIP " EVALUATE ; IMMEDIATE

CR .( Loading benchmark routines)

( Benchmark support words)

: .### ( n)   <#  # # #  '.' HOLD  #S  #>
   7 OVER - SPACES  TYPE  SPACE ;

\ TIMER  is redefined to format the elapsed time better.

: TIMER ( n n)
        s>d d>f counter swap - s>d d>f 1e3 f* fswap f/
        fdup 1e3 f<
        if      f. ." us"
        else    fdup 1e6 f<
                if      1e3 f/ f. ." ms"
                else    1e6 f/ f. ." sec"
                then
        then    ;

: [$ ( - n)   CR  ." Testing "  COUNTER ;

\ $]  is the suffix to a testing word.  It takes the fast ticks
\    timer value and calculates the elapsed time.  It does do
\    some display words before calculating the time, but it is
\    assumed that this will take minimal time to execute.

: $] ( n)   ."  = "  TIMER ;

\ CARRAY  creates a byte size array.
: CARRAY ( n)   CREATE  ALLOT  
		DOES> ( n - a)  + ;

\ ARRAY  creates a word size array.
: ARRAY ( n)    CREATE  CELLS ALLOT  
		DOES> ( n - a) SWAP CELLS + ;

( Basic FORTH, Inc. Benchmarks)
\ This series of tests analyze the Forth primatives.

: $DO$   [$  ." DO LOOP"  1000000 DUP 0 DO  I DROP  LOOP  $] ;
: $*$   [$  ." *"  1000000 DUP    0 DO  I I *    DROP LOOP  $] ;
: $/$   [$  ." /"  1000000 DUP 1+ 1 DO  1000 I / DROP LOOP  $] ;
: $+$   [$  ." +"  1000000 DUP 1+ 1 DO  1000 I + DROP LOOP  $] ;
: $M*$   [$ ." M*" 1000000 DUP    0 DO I I M*     2DROP LOOP $] ;
: $M/$   [$ ." M/" 1000000 DUP 1+ 1 DO 1000 0 I M/  DROP LOOP $] ;
: $M+$   [$ ." M+" 1000000 DUP 1+ 1 DO 1000 0 I M+ 2DROP LOOP $] ;
: $/MOD$   [$  ." /MOD"  1000000 DUP 1+ 1 DO  1000 I /MOD  2DROP LOOP $] ;

\ $*/$  tests the math primative  */ .  This may or may not tell
\    you how the other math primatives perform depending on
\    how  */  has been coded.

: $*/$   [$ ." */"  1000000 DUP 1+ 1 DO  I I I */ DROP  LOOP $] ;

( Eratosthenes sieve benchmark program )

\ This is NOT the original BYTE benchmark.

8190 CONSTANT SIZE       SIZE CARRAY FLAGS

: DO-PRIME     
      10 0 DO
	       0 FLAGS SIZE -1 FILL
               0 SIZE 0
               DO I FLAGS C@
                    IF I 2* 3 + DUP I +
                         BEGIN DUP SIZE <
                         WHILE DUP FLAGS 0 SWAP C! OVER +
                         REPEAT 2DROP
                              1+
                    THEN
               LOOP DROP 
	 LOOP ." 1899 Primes"  ;

: $SIEVE$   [$  ." Eratosthenes sieve "  DO-PRIME  SIZE 10 * $] ;

( QuickSort from Hoare & Wil Baden)

7 CELLS CONSTANT THRESHOLD

: Precedes  ( n1 n2 - f)   U< ;
: Exchange  ( a1 a2)   2DUP  @ SWAP @ ROT !  SWAP ! ;
: Both-Ends  ( f l pivot - f l )
    >R  BEGIN   OVER @ R@ Precedes
        WHILE  CELL 0 D+   REPEAT
        BEGIN   R@ OVER @ Precedes
        WHILE  CELL -      REPEAT   R> DROP ;

: Order3  ( f l - f l pivot)   2DUP OVER - 2/ -CELL AND + >R
      DUP @ R@ @ Precedes  IF DUP R@ Exchange THEN
      OVER @ R@ @ SWAP Precedes
        IF OVER R@ Exchange  DUP @ R@ @ Precedes
          IF DUP R@ Exchange THEN  THEN   R>  ;

: Partition  ( f l - f l' f' l)   Order3 @ >R  2DUP
      CELL -CELL D+  BEGIN    R@ Both-Ends 2DUP 1+ U<
      IF  2DUP Exchange CELL -CELL D+  THEN
      2DUP SWAP U<   UNTIL R> DROP SWAP ROT ;

: Sink  ( f key where - f)   ROT >R
   BEGIN   CELL - 2DUP @ Precedes
   WHILE  DUP @ OVER CELL + !  DUP R@ =
        IF  ! R> EXIT THEN   ( key where)
   REPEAT  CELL + ! R> ;

: Insertion  ( f l)   2DUP U<
    IF  CELL + OVER CELL +   DO  I @ I Sink  CELL +LOOP DROP
    ELSE  ( f l) 2DROP  THEN ;

: Hoarify  ( f l - ...)
    BEGIN   2DUP THRESHOLD 0 D+ U<
    WHILE  Partition  2DUP - >R  2OVER - R> >  IF 2SWAP THEN
    REPEAT Insertion ;

: QUICK  ( f l)   DEPTH >R   BEGIN  Hoarify DEPTH R@ <
      UNTIL  R> DROP ;

: SORT  ( a n)   DUP 0= ABORT" Nothing to sort "
    1- CELLS  OVER +  QUICK ;

ALIGN  30000 ARRAY POINTERS

: fill   30000 0 DO  (* 30000 ( I') I - *) I I POINTERS !  LOOP ;

: (sort)  10 0 DO   fill  0 POINTERS 1000 SORT   LOOP ;

: $SORT$   [$  ." Hoare's quick sort (reverse order)"
   	       (sort) 10 1000 * $] ;

: Z ( n)   PAGE  0 DO  I POINTERS @  8 U.R  LOOP ;

: BENCHMARK   PAGE  [$  ." the "  ." iForth" ( .cold ) ."  system"  CR
   $DO$ $*$  $/$  $+$  $M*$  $M/$  $M+$  $/MOD$  $*/$  $SIEVE$ $SORT$
   CR  CR  ." To run all tests"  1 $] ;

BENCHMARK
