(*
 * LANGUAGE    : ANS Forth 
 * PROJECT     : Forth Environments
 * DESCRIPTION : "Forth-like DHRYSTONE"
 * CATEGORY    : Benchmarks
 * AUTHOR      : Marcel Hendrix, August 3rd, 1991
 * LAST CHANGE : October 5th, 1991 MHX
 *)



	NEEDS -miscutil

	REVISION -dhrystone " Dhrystone Benchmark Version 2.00 "

	PRIVATES


DOC
(*

"DHRYSTONE" Benchmark Program

Version: Forth/1
Date:	 05/03/86
Author:	 Reinhold P. Weicker, CACM Vol 27, No 10, 10/84 pg. 1013

	C version translated from ADA by Rick Richardson.
	Every method to preserve ADA-likeness has been used,
	at the expense of C-ness.
	Modula-2 version translated from C by Kevin Northover.
	Again every attempt made to avoid distortions of the original.
	Forth version translated from Modula-2 by Marcel Hendrix.
	Distorting the original was inevitable, given the differences
	between a strongly typed and a user-extensible language.
	Moreover, there is serious doubt of the instruction mix being
	appropriate for Forth.

	The following program contains statements of a high-level
	programming language (Forth) in a distribution considered 
	representative:

	statements			53%
	control statements		32%
	procedures, function calls	15%

	100 statements are dynamically executed. The program is balanced
	with respect to the three aspects:

		- statement type
		- operand type (for simple data types)
		- operand access
		    operand global, local parameter, or constant.

	The combination of these three aspects is balanced only
	approximately.

	The program does not compute anything meaningful, but it is
	syntactically and semantically correct.

	The source code was "pre-optimized" on a word-to-word basis with
	the programmer acting as a pre-processor to the compiler.

	Real Forth programmers would rather be found dead than 
	write disgusting programs like this.

	If you understand what both DHRYSTON.C and DHRYSTON.FRT are doing, 
	you'll never trust a benchmark again (unless you wrote it yourself).
*)
ENDDOC


#100000 VALUE LOOPS	-- Control human fatigue factor

	-- Some types

1 =: Ident1	PRIVATE
2 =: Ident2	PRIVATE
3 =: Ident3	PRIVATE
4 =: Ident4	PRIVATE
5 =: Ident5	PRIVATE



CREATE	Array1Glob	#50 	   CELLS ALLOT 	PRIVATE
CREATE	Array2Glob	#50 DUP *  CELLS ALLOT 	PRIVATE

\ 1 REGISTER sav PRIVATE

: []Array1Par	 EVAL" Array1Par []CELL " ;		IMMEDIATE PRIVATE
: [][]Array2Par  EVAL" #50 * + Array2Par []CELL " ;	IMMEDIATE PRIVATE
: ADDRESS	 ; 					IMMEDIATE PRIVATE



0 VALUE /bytes	PRIVATE

: RECORD	CREATE	CLEAR /bytes  HERE 0 ,	\ <> --- <sys>
		DOES>	@ ALLOCATE ?ALLOCATE	\ <> --- <addr>
			IMMEDIATE ; PRIVATE


: END		/bytes SWAP ! ; PRIVATE		\ <sys> --- <>

: SIMPLE-TYPE   CREATE	,			\ <fieldlength> --- <>
		DOES>	@ 
			CREATE	IMMEDIATE
				/bytes ,
				+TO /bytes
			DOES>	@		\ <'record> --- <'offset>
				EVAL" LITERAL + " ; PRIVATE


 1  CELLS SIMPLE-TYPE	RecordPtr	PRIVATE
 1  CELLS SIMPLE-TYPE	Enumeration	PRIVATE	-- one of Ident1 .. Ident5
 1  CELLS SIMPLE-TYPE	OneToFifty	PRIVATE
#31 CHARS SIMPLE-TYPE	String30	PRIVATE	-- extra count byte


RECORD RecordType			PRIVATE	-- offset
		RecordPtr    PtrComp	PRIVATE	-- 0
		Enumeration  Discr	PRIVATE	-- 1 CELLS
		Enumeration  EnumComp	PRIVATE	-- 2 CELLS
		OneToFifty   IntComp	PRIVATE	-- 3 CELLS
		String30     StringComp	PRIVATE	-- 4 CELLS
END 


	-- Some global variables

0 VALUE IntGlob		PRIVATE
0 VALUE BoolGlob	PRIVATE
0 VALUE Char1Glob 	PRIVATE
0 VALUE Char2Glob 	PRIVATE


0 =: NIL		PRIVATE

NIL VALUE PtrGlb	PRIVATE
NIL VALUE PtrGlbNext	PRIVATE



: Proc7		EVAL" + 2+ " ; IMMEDIATE	\ <n1> <n2> --- <n3>
		PRIVATE

: Proc3		PtrGlb IF PtrGlb PtrComp @ 	\ <'record> --- <>
			  SWAP !
		     ELSE DROP #100 TO IntGlob
		    ENDIF
		#10 IntGlob  Proc7   PtrGlb IntComp ! ; PRIVATE


: Func3		EVAL" Ident3 = " ; IMMEDIATE PRIVATE


: Proc6		LOCAL n2  LOCAL n1  n1 LOCAL n	\ <n1> <n2> --- <n>
		n1 Func3 0= IF Ident4 TO n
			 ENDIF
		n1 CASE 
		    Ident1 OF Ident1 		ENDOF
		    Ident2 OF IntGlob
			      #100 > IF Ident1 
				   ELSE Ident4 
				  ENDIF 	ENDOF
		    Ident3 OF Ident2 		ENDOF
		    Ident4 OF n			ENDOF
		    Ident5 OF Ident3 		ENDOF
		    ABORT" Proc6: argument out of range"
		   ENDCASE ; PRIVATE


0 REGISTER p^	PRIVATE

: Proc1		TO p^				\ <'record> --- <>
		PtrGlb  p^ PtrComp !
		5	p^ IntComp !
		p^ IntComp @   p^ PtrComp @ IntComp !
		p^ PtrComp @   p^ PtrComp @ PtrComp @  !
		p^ PtrComp @ PtrComp @ Proc3
		p^ PtrComp @ Discr   @ Ident1
		 = IF 6  p^ PtrComp @ IntComp  !
		      p^ PtrComp @ EnumComp  p^ EnumComp @  OVER @ Proc6 SWAP !
		      PtrGlb PtrComp  p^ PtrComp @ PtrComp !
		      p^ PtrComp @ IntComp DUP @  #10  Proc7 SWAP !
		 ELSE p^ PtrComp @ p^ !
		ENDIF ; PRIVATE


: Proc2		DUP #10 + LOCAL IntLoc		\ <val> --- <val'>
		BEGIN	Char1Glob 'A' =		\ This one never ends
		WHILE	-1 +TO IntLoc		\ unless Char = 'A' ??
			DROP IntLoc IntGlob - 
			TRUE
		UNTIL	THEN ; PRIVATE


: Proc4		EVAL" 'B' TO Char2Glob " ; 		      IMMEDIATE PRIVATE
: Proc5		EVAL" 'A' TO Char1Glob  FALSE TO BoolGlob " ; IMMEDIATE PRIVATE

: Proc8		    LOCAL IntParI2	\ <'array1> <'array2> <n1> <n2> --- <>
		5 + LOCAL IntLoc  
		    LOCAL Array2Par
		    LOCAL Array1Par
		
		IntLoc []Array1Par IntParI2 OVER ! 
		( addr) @  IntLoc 1+ []Array1Par !
		IntLoc   DUP   #30 + []Array1Par !
\		IntLoc 2+  IntLoc DO  IntLoc 
\				      DUP I [][]Array2Par !
\				LOOP
		IntLoc DUP DUP    [][]Array2Par !
		IntLoc DUP DUP 1+ [][]Array2Par !

		1  IntLoc DUP 1-  [][]Array2Par +! 
		IntLoc []Array1Par @   
		  IntLoc DUP #20 + SWAP [][]Array2Par !
		5 TO IntGlob ; PRIVATE


: Func1		\ <char1> <char2> --- <n>
		EVAL" = IF Ident2 ELSE Ident1 ENDIF " ; IMMEDIATE PRIVATE


: Func2		   LOCAL '$2  			\ <'$1> <'$2> --- <bool>
		   LOCAL '$1
		 2 LOCAL IntLoc
		BL LOCAL CharLoc

		BEGIN  IntLoc 2 <=
		WHILE  IntLoc 1+ '$1 + C@   
		       IntLoc 2+ '$2 + C@  Func1
			Ident1 = IF 'A' TO CharLoc  
			             1 +TO IntLoc
			      ENDIF
		REPEAT

		CharLoc 'W' >= 
		   IF  CharLoc 'Z' <= 
		         IF 7 TO IntLoc		\ dead code, IntLoc never used!
		      ENDIF
		ENDIF

		CharLoc 'X' = IF TRUE EXIT ENDIF

		'$1 COUNT '$2 COUNT COMPARE
		0> 
		   IF 7 +TO IntLoc TRUE		\ dead code, IntLoc is local
		 ELSE FALSE
		ENDIF ; PRIVATE



: Proc0		0 LOCAL IntLoc1
		0 LOCAL IntLoc2
		0 LOCAL IntLoc3
		#31 ALLOCATE ?ALLOCATE LOCAL String1Loc
		#31 ALLOCATE ?ALLOCATE LOCAL String2Loc
		0 LOCAL EnumLoc
		0 LOCAL CharLoc
		0 LOCAL CharIndex
		RecordType TO PtrGlb		\ constructor, allocates !
		RecordType TO PtrGlbNext
		PtrGlbNext  PtrGlb PtrComp  ! 
		Ident1      PtrGlb Discr    ! 
		Ident3      PtrGlb EnumComp ! 
		#40	    PtrGlb IntComp  ! 
		C" DHRYSTONE PROGRAM, SOME STRING" 
		DUP C@ 1+  PtrGlb StringComp  SWAP MOVE
   LOOPS 0 DO	
		Proc5  
		Proc4
		2 TO IntLoc1  3 TO IntLoc2
		C" DHRYSTONE PROGRAM, 2'ND STRING" 
		DUP C@ 1+ String2Loc SWAP MOVE
		Ident2 TO EnumLoc
		String1Loc String2Loc  Func2 INVERT  TO BoolGlob  

		BEGIN  
		   IntLoc1 IntLoc2 <
		WHILE  
		   IntLoc1 5 * IntLoc2 - TO IntLoc3
		   IntLoc1 IntLoc2 Proc7 TO IntLoc3	\ The Forth way
		   1 +TO IntLoc1
		REPEAT

		ADDRESS Array1Glob ADDRESS Array2Glob IntLoc1 IntLoc2  Proc8
		PtrGlb Proc1
		'A' TO CharIndex

		BEGIN  
		  CharIndex Char2Glob <=
		WHILE  
		  CharIndex 'C' Func1
		  EnumLoc = IF  Ident1 EnumLoc Proc6 TO EnumLoc
		         ENDIF
		  1 +TO CharIndex	
		REPEAT

		IntLoc1 IntLoc2 * TO IntLoc3
		IntLoc3 IntLoc1 / TO IntLoc2
		IntLoc3 IntLoc2 - 7 * IntLoc1 - TO IntLoc2
		IntLoc1 Proc2 TO IntLoc1		\ the Forth way
	 LOOP	
	 	PtrGlb     FREE ?ALLOCATE
		PtrGlbNext FREE ?ALLOCATE
		String1Loc FREE ?ALLOCATE
		String2Loc FREE ?ALLOCATE ; PRIVATE


	-- The main program is trivial

: DHRYSTONE	CR ." Starting Dhrystone, " LOOPS DEC. ." loops ... "
		TIMER-RESET  Proc0  .ELAPSED
		CR ." That is " 
		  LOOPS #1000 diff0 @ */ DEC. ." Dhrystones/sec." ;


: +CR		CR #10 SPACES ; PRIVATE

:ABOUT		CR CR
		+CR ." -- DHRYSTONE II performance test --" CR
		+CR ."            Typical values:         "
		+CR ."    ('386+'387 at 33 MHz, cache on) " 
		+CR ."              ('486 at 66 MHz) " 
		+CR ."      (Transputer: T800 at 20 MHz)  " CR
		+CR ." Par.C   3L   TC 1.5  iForth3  iForth4  tForth"
		+CR ." ---------------------------------------------"
		+CR ."  3333  5555   8333    6734     33101    4403 " CR
		+CR ." Units: Dhrystones/second."
		+CR ." Enter 'DHRYSTONE' to test." 
		+CR ." (this tests integer performance)" ;


		DEPRIVE

		.ABOUT -dhrystone

			    (* End of Source *)