(*
 * LANGUAGE    : ANS Forth
 * PROJECT     : Forth Environments
 * DESCRIPTION : Benchmarks from Forth Dimensions March/April '92, G. Kelly
 * CATEGORY    : Benchmark
 * AUTHOR      : Marcel Hendrix
 * LAST CHANGE : July 12, 1994, Marcel Hendrix; interpreter/compiler tests
 * LAST CHANGE : February 2, 1993, Marcel Hendrix
 *)



	NEEDS -miscutil

	REVISION -fd92 " FD Vol XIII Bench   Version 0.02 "



\ Empty loop	Empty = XX

: X		30000 0 DO   LOOP ;
: XX		    5 0 DO X LOOP ;


\ Threading	Thread = YY - XX

S" TFORTH" ENVIRONMENT?
	   [IF] DROP  		      CODE NC 	gcall,		      END-CODE
	 [ELSE]	S" IFORTH" ENVIRONMENT?
			   [IF] DROP  CODE NC ( NEXT, ) $FF C, $E3 C, END-CODE
			 [ELSE]	      CODE NC   NEXT,  		      END-CODE
			 [THEN]
	 [THEN]

: Y		30000 0 DO  NC NC NC  NC NC NC  LOOP ;
: YY		    5 0 DO  Y  LOOP ;


\ Nesting1	Nest1 = ZZ - XX

: N:	;
: Z		30000 0 DO  N: N: N:  N: N: N:  LOOP ;
: ZZ		    5 0 DO  Z  LOOP ;


\ Nesting2	Nest2 = WW - XX

: W1 	   ;
: W2 	W1 ;
: W3 	W2 ;
: W4 	W3 ;
: W5 	W4 ;
: W6 	W5 ;

: W		30000 0 DO  W6  LOOP ;
: WW		    5 0 DO  W   LOOP ;


\ Primitives 	Prims = QQ - XX
\ Exercise: variable constant @ ! + DUP SWAP OVER DROP

   VARIABLE LOC
10 CONSTANT TEN

: NULL		TEN DUP  LOC SWAP  OVER !  @ +  DROP ;
: Q		30000 0 DO  NULL  LOOP ;
: QQ		    5 0 DO  Q     LOOP ;


\ Standard Sieve =  10 0 do  do-prime  loop

8190 CONSTANT size
     CREATE   flags  	size ALLOT

: DO-PRIME	flags size 1 FILL
		0  size 0 DO flags I +
			     C@ IF I DUP +  3 +
				   DUP  I +
			  	    BEGIN  DUP size <
				    WHILE  0 OVER flags + C!
					   OVER +
				    REPEAT
			 	   2DROP 1+
			     ENDIF
			LOOP  ( count) DROP ;

: RR		10 0 DO DO-PRIME LOOP ;

: S1		S" testfile.frt" INCLUDED ;
: S2		S" testf2.frt"   INCLUDED ;
: S3		S" testf3.frt"   INCLUDED ;
: S4		S" testf4.frt"   INCLUDED ;
: S5		S" testf5.frt"   INCLUDED ;
: S6		S" testf6.frt"   INCLUDED ;
: S7		S" testf7.frt"   INCLUDED ;
: S8		S" testf8.frt"   INCLUDED ;
: S9		S" testf9.frt"   INCLUDED ;

: DO-TEST	CR ." *** Benchmark from FD March/April 1992, Guy Kelly ***"
		S" ?DEF -rubbish [IF] -rubbish [THEN] MARKER -rubbish" EVALUATE
		CR ."    Empty      : " TIMER-RESET XX .ELAPSED
		CR ."    Thread     : " TIMER-RESET YY .ELAPSED
		CR ."    Nest1      : " TIMER-RESET ZZ .ELAPSED
		CR ."    Nest2      : " TIMER-RESET WW .ELAPSED
		CR ."    Prims      : " TIMER-RESET QQ .ELAPSED
		CR ."    Sieve      : " TIMER-RESET RR .ELAPSED
		CR ."    Loads      : " TIMER-RESET S1 .ELAPSED
		CR ."    Comp       : "             S2
		CR ."    C prim     : "             S3
		CR ."    C sec      : "             S4
		CR ."   rd+wrd+fnd  : " TIMER-RESET S5 .ELAPSED
 		CR ." read + <word> : " TIMER-RESET S6 .ELAPSED
		CR ."    <word>     : " TIMER-RESET S7 .ELAPSED
		CR ."     word      : " TIMER-RESET S8 .ELAPSED
		CR ."     refill    : " TIMER-RESET S9 .ELAPSED ;


:ABOUT	CR ." ***     Benchmark from FD March/April 1992,  Guy Kelly     ***"
	CR ." ***     Additional tests added July 1994, Marcel Hendrix   ***"
	CR ." ***  Results on a '486 @ 66 MHz, T @ 25 MHz, dsp @ 30 MHz  ***"
	CR ." ***  For the transputer, 'F' is F-server, 'C' is C-server  ***"
	CR ." ***  First column: Pentium 166 MHz running iForth 1.06     ***"
	CR
	CR ."     BENCH       P5-i4     iForth     F-PC    tForth  F / C  dspForth  CHForth"
	CR ." -----------------------------------------------------------------------------"
	CR ."     Empty     : 0.005     0.038     0.050    0.188          0.070     0.054 "
	CR ."     Thread    : 0.058     0.122     0.220    0.821          0.609     0.164 "
	CR ."     Nest1     : 0.080     0.268     0.830    1.563          0.070     0.659 "
	CR ."     Nest2     : 0.015     0.071     0.770    0.614          0.070     0.604 "
	CR ."     Prims     : 0.045     0.136     0.540    0.720          0.339     0.439 "
	CR ."     Sieve     : 0.025     0.152     0.660    0.820          0.684     0.604 "
	CR ."     Loads     : 0.007     0.062     0.060    0.256            --      0.109 "
	CR ."     Comp      : 0.008     0.069     0.060    0.454            --      0.164 "
	CR ."     C prim    : 0.187     0.689     0.280    9.290 /  9.812   --      1.428 "
	CR ."     C sec     : 0.180     0.688     0.220    ?.??? /  ?.???   --      1.263 "
	CR ."   rd+wrd+fnd  : 0.102     0.469     0.160   20.100 / 29.541   --      1.868 "
	CR ." read + <word> : 0.027     0.147     0.060   12.910 / 22.771   --      0.874 "
	CR ."     <word>    : 0.217     0.817     1.100   20.020 / 18.751   --      1.318 "
	CR ."      word     : 0.243     0.982     1.100   22.090 / 20.779   --      1.317 "
	CR ."     refill    : 0.084     0.490     0.110   ??.??? / ??.???   --       --   " ;


FALSE VALUE caps?	\ TRUE -> 10 to 20% faster

                .ABOUT -fd92 CR
		MARKER -rubbish

                              (* End of Source *)