(*
 * LANGUAGE    : ANS Forth 
 * PROJECT     : Forth Environments
 * DESCRIPTION : "Forth-like WHETSTONE"
 * CATEGORY    : Benchmarks
 * AUTHOR      : Marcel Hendrix, July 31th, 1991
 * LAST CHANGE : Jan 1st, 1992 MHX, optimized p3
 * LAST CHANGE : October 5th, 1991 MHX
 *)



	NEEDS -miscutil
	NEEDS -arrays

	REVISION -whetstone " Whetstone Benchmark Version 1.10 "

	PRIVATES


DOC
(*
      Whetstone benchmark in Forth.  This program is a translation of the
    C translation of the original Algol version in "A Synthetic Benchmark" 
    by H.J. Curnow and B.A. Wichman in Computer Journal, Vol  19 #1,
    February 1976.

    Used to test compiler optimization and floating point performance.

    Compile by:		[TRUE =: testing] [TRUE =: timing] INCLUDE whetston.frt

    To debug:			 TRUE =: testing
    To time individual modules:  TRUE =: timing
*)
ENDDOC


?DEF timing 
  [IF]
	: t(	TIMER-RESET ; PRIVATE
	: )t	CR .ELAPSED 	\ <ix> --- <ix'>
		."  (" DUP 0 .R ')' EMIT 1+ ; PRIVATE
[ELSE]	
	: t(	; IMMEDIATE PRIVATE
	: )t	; IMMEDIATE PRIVATE
[THEN]


?DEF testing 
  [IF]
	: pout		\ <n1> <n2> <n3> --- <> \F <r1> <r2> <r3> <r4> --- <>
		BASE @ >S DECIMAL
		  CR ROT 5 .R   SWAP 5 .R   5 .R  3 SPACES 
		  FSWAP 3 FPICK 6 +E.R SPACE  
		  FROT 6 +E.R SPACE   6 +E.R SPACE  6 +E.R FDROP
		S> BASE ! ; PRIVATE
[THEN]


	S" EMULATED" ENVIRONMENT? AND 
	  [IF] 2 
	[ELSE] #2000 
	[THEN] =: ITERATIONS 	-- 200 Million Whetstone instructions


  0.499975e	FCONSTANT t	PRIVATE
  0.50025e	FCONSTANT t1	PRIVATE
  2.0e		FCONSTANT t2	PRIVATE
  1.0e		FCONSTANT 1E	PRIVATE

    (* set values of module weights *)

       0 ITERATIONS * =: n1	-- [integer] module is skipped
     #12 ITERATIONS * =: n2
     #14 ITERATIONS * =: n3
    #345 ITERATIONS * =: n4
   ( ??? ITERATIONS * =: n5 )	-- was not included in whetston.c
    #210 ITERATIONS * =: n6
     #32 ITERATIONS * =: n7
    #899 ITERATIONS * =: n8
    #616 ITERATIONS * =: n9
       0 ITERATIONS * =: n10	-- [integer] module is skipped
     #93 ITERATIONS * =: n11


4 FARRAY e1[]	PRIVATE



	(* MODULE 1:  simple identifiers [n1 = 0!] *)

: MODULE1	 1.0e FLOCAL x1
		-1.0e FLOCAL x2
		   x2 FLOCAL x3 
		   x3 FLOCAL x4

		n1 ?FOR
			    x1 x2 F+  x3 F+  x4 F-  t F*  TO x1
			    x1 x2 F+  x3 F-  x4 F-  t F*  TO x2
			    x1 x2 F-  x3 F+  x4 F+  t F*  TO x3
			    x2 x1 F-  x3 F+  x4 F+  t F*  TO x4
		   NEXT 

 [ ?DEF testing ] [IF]  n1 n1 n1  x1 x2 x3 x4  pout
		[THEN] ;



	(* MODULE 2:  array elements *)

: MODULE2	 1.0e TO 0 e1[]
		-1.0e TO 1 e1[]
		-1.0e TO 2 e1[]		
		-1.0e TO 3 e1[]

		n2 ?FOR
		        0 e1[] 1 e1[] F+  2 e1[] F+  3 e1[] F-  t F*  TO 0 e1[]
		        0 e1[] 1 e1[] F+  2 e1[] F-  3 e1[] F+  t F*  TO 1 e1[]
		        0 e1[] 1 e1[] F-  2 e1[] F+  3 e1[] F+  t F*  TO 2 e1[]
		        1 e1[] 0 e1[] F-  2 e1[] F+  3 e1[] F+  t F*  TO 3 e1[]
		   NEXT 

 [ ?DEF testing ] [IF]  n2 n3 n2   0 e1[]  1 e1[]  2 e1[]  3 e1[]  pout
		[THEN] ;



	(* MODULE 3:  array as parameter *)


-- Special routine to cope with an array passed by address.

: a[]		%VAR @  0 %VAR !	\ run: <index> --- <?>
		CASE 			
		( +to)	 -1 OF EVAL" addr []FLOAT F+! " ENDOF
		( from)	  0 OF EVAL" addr []FLOAT F@  " ENDOF
		( to)	  1 OF EVAL" addr []FLOAT F!  " ENDOF
		( 0to)	  2 OF EVAL" addr []FLOAT F0! " ENDOF
			  DUP ABORT" a[]: unknown message"
		ENDCASE ; IMMEDIATE PRIVATE

0 REGISTER addr	PRIVATE


: pa		TO addr					\ <addr> --- <>
		6 ?FOR 
		    0 a[] 1 a[] F+   2 a[] F+   3 a[] F-  t F*   TO 0 a[]
		    0 a[] 1 a[] F+   2 a[] F-   3 a[] F+  t F*   TO 1 a[]
		    0 a[] 1 a[] F-   2 a[] F+   3 a[] F+  t F*   TO 2 a[]
		    1 a[] 0 a[] F-   2 a[] F+   3 a[] F+  t F*   TO 3 a[]
		  NEXT ; PRIVATE


: MODULE3	n3 ?FOR
		      'OF e1[] pa
		   NEXT

 [ ?DEF testing ] [IF]  n3 n2 n2   0 e1[]  1 e1[]  2 e1[]  3 e1[]  pout
		[THEN] ;



	(* MODULE 4:  conditional jumps *)

: MODULE4	1
		n4 ?FOR
		        1 = 3 +    ( IF 2 ELSE 3 ENDIF)
		        2 > 1+     ( IF 0 ELSE 1 ENDIF)
		        1 < NEGATE ( IF 1 ELSE 0 ENDIF)
		   NEXT

 [ ?DEF testing ] [IF]  DUP n4 -ROT  PI PI PI PI pout
		[ELSE]  DROP
		[THEN] ;



	(* MODULE 5:  omitted [whetston.c didn't show it] *)

: MODULE5 ; IMMEDIATE


	(* MODULE 6:  integer arithmetic; type conversion *)

1 REGISTER j	PRIVATE
2 REGISTER k	PRIVATE
3 REGISTER l	PRIVATE

: MODULE6	1 TO j	
		2 TO k	
		3 TO l

		n6 ?FOR
			k j -  l k -  *  j *    TO j
			l k *  l j -     k *  - TO k
			l k -  k j +  * 	TO l

			j k + l +  S>F  TO (( l 2- )) e1[]
			j k * l *  S>F  TO (( k 2- )) e1[]
		   NEXT 

 [ ?DEF testing ] [IF]  n6 j k   0 e1[]  1 e1[]  2 e1[]  3 e1[]  pout
		[THEN] ;


	(* MODULE 7:  trig. functions *)

: MODULE7	0.5e FLOCAL x
		 x   FLOCAL y

		n7 ?FOR
			x y F+ FCOS   x y F- FCOS  F+  F1-
			x FSINCOS F* t2 F*  
			FOVER F/ FATAN  t F*  TO x

			y FSINCOS F* t2 F*  
			FSWAP F/ FATAN  t F*  TO y
		   NEXT

 [ ?DEF testing ] [IF]  n7 0 0   x x y y  pout
		[THEN] ;


	(* MODULE 8:  procedure calls *)


S" EMULATED" ENVIRONMENT? DROP
[IF]

: p3		F+ t F*  	 	\ <'r3> --- <> \F <r1> <r2> --- <>
		FDUP F+  t2 F/  F! ; PRIVATE

[ELSE]

CREATE 't2  	t F2*  t2 F/  F,   PRIVATE

S" TFORTH" ENVIRONMENT? 
    [IF] DROP
	CODE p3			\ <'r3> --- <> \F <r1> <r2> --- <>
		>pr0,
		fpop, fpop, fpadd,
		't2 *ldpi, 
	F#64	fpldnldb,	
	F#32	fpldnlsn,	
		fpmul,
		dpop,
	F#64 	fpstnldb,
	F#32 	fpstnlsn,
		pr0>, gcall,
	END-CODE PRIVATE
  [ELSE]
	: p3	EVAL" F+  't2 F@ F*  F! " ; \ <'r3> --- <> \F <r1> <r2> --- <>
		IMMEDIATE PRIVATE
  [THEN]	

[THEN]



: MODULE8	1.0e FLOCAL x
		  x  FLOCAL y
		  x  FLOCAL z

		n8 ?FOR
			 x y 'OF z p3
		   NEXT

 [ ?DEF testing ] [IF]  n8 0 0   x y z FDUP  pout
		[THEN] ;



	(* MODULE9:  array references *)


: MODULE9	1 TO j
		2 TO k
		3 TO l

		1.0e TO 0 e1[]
		2.0e TO 1 e1[]
		3.0e TO 2 e1[]

		n9 ?FOR 
			k e1[]  TO (( j )) e1[]
			l e1[]  TO (( k )) e1[]
			j e1[]  TO (( l )) e1[]
		   NEXT

 [ ?DEF testing ] [IF]  n9 j k   0 e1[]  1 e1[]  2 e1[]  3 e1[]  pout
		[THEN] ;



	(* MODULE10:  integer arithmetic [n10 = 0!] *)

: MODULE10	2 TO j
		3 TO k

		n10 ?FOR
			k +TO j
			j +TO k
			k j - TO j
			j 2* NEGATE +TO k
		    NEXT 

 [ ?DEF testing ] [IF]  n10 j k   PI PI PI PI  pout
		[THEN] ;



	(* MODULE11:  standard functions *)

: MODULE11	0.75e
		n11 ?FOR
			FLOG  t1 F/  FEXP  FSQRT
		    NEXT

 [ ?DEF testing ] [IF]  n11 0 0   FDUP FDUP FDUP pout
		[ELSE] FDROP
		[THEN] ;



: WHETSTONE	CR ."  Start ..."
		TIMER-RESET 

		1 			\ sequence number
		  t( MODULE1  )t
		  t( MODULE2  )t
		  t( MODULE3  )t 
		  t( MODULE4  )t
		  t( MODULE5  )t
	  	  t( MODULE6  )t
		  t( MODULE7  )t
		  t( MODULE8  )t
		  t( MODULE9  )t
		  t( MODULE10 )t
		  t( MODULE11 )t
		DROP 			\ drop sequence number

[ ?UNDEF testing ?UNDEF timing AND ]
[IF]		CR .ELAPSED
		CR ." That makes " 
		ITERATIONS #100000 diff0 @  */  DEC. ." KWhet/sec." CR 
[THEN] 		
		;


:ABOUT	CR CR
	CR ." --------------- WHETSTONE performance test --------------" CR
	CR ." To debug, define 'TRUE =: testing' _before_ loading      " CR
	CR ." To time MODULES, define 'TRUE =: timing' _before_ loading" CR
	CR ." Typical values for various machines & compilers:" 
	CR ." (TC & iForth3: '386+387 at 33 MHz, cache on, doubles)"
	CR ."              (iForth4: '486 at 66 MHz)"
	CR ."       (Transputer: T800 @ 20 MHz, doubles)" 
	CR ."       Par.C   3L   TC 1.5  iForth3/4  tForth "
	CR ."       -------------------------------------- "
	CR ."       1720  1754    4560   3305/13289   2834 " CR
	CR ." Units: KWhet/second
	CR ." Enter 'WHETSTONE' to test." 
	CR ." (this tests floating-point performance)" ;


		DEPRIVE

		.ABOUT -whetstone CR

			    (* End of Source *)